//////////////////////////////////////////////////
// Author: Parijat Banerjee, Sourabh Dube
// Date: Jan 24th, 2022
// Classification of Gen Events into one of
// the seven orthogonal multilepton final
// state channels
//////////////////////////////////////////////////
#define delZAna_cxx

#include "delZAna.h"
#include <TH2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <iostream>
#include <TRandom3.h>
#include <TMath.h>
void delZAna::Begin()
{
  //Initialize global variables here.
  GEV = 1000.;
  MEV2GEV = .001;
  nEvtTotal = 0;
  njet=0;
  nb=0;
  nossf=0;
  nj=0;
  dr_ll=1.0;
  dRmin_OR_Nj=0;
  chan_ctr[7]={0};
  ctr[43]={0};
  //Create the histogram file
  _HstFile = new TFile(_HstFileName,"recreate");
  //Call the function to book the histograms we declared in Hists.
  BookHistograms();
  std::cout<<"Efficiency map running is "<<_EffFileName<<std::endl;
  effmap = new TFile(_EffFileName);
}

void delZAna::End()
{
  //Write histograms and close histogram file
  _HstFile->Write();
  _HstFile->Close();
  //Output to screen.
  cout<<"Total events = "<<nEvtTotal<<endl;
  //Open the text output file
  ofstream fout(_SumFileName);
  //Put text output in the summary file.
  fout<<"Total events = "<<nEvtTotal<<endl;
  for(int i=0;i<43;i++){
    if(ctr[i]>0){
      fout<<iCat_name[i]<<" : "<<ctr[i]<<endl;
    }
  }
  for(int i=0;i<7;i++){
    fout<<iChan_name[i]<<" :"<<chan_ctr[i]<<endl;
  }
}

void delZAna::Loop()
{
  //
  // This function should contain the "body" of the analysis. It can contain
  // simple or elaborate selection criteria, run algorithms on the data
  // of the event and typically fill histograms.
  //
  
  if(fChain==0) return;
  Long64_t numberOfEntries = treeReader->GetEntries();
  //This prints total entries before processing begins
  cout<<"Total entries = "<<numberOfEntries<<endl;
  nEvtTotal=0;
  
  //This is the main loop over the number of events
  for(Int_t entry = 0; entry < numberOfEntries; ++entry){
    
    treeReader->ReadEntry(entry);
    nEvtTotal++;
    //Output processing information to screen based on verbosity level.
    if(_verbosity>5000)cout<<"Processed "<<nEvtTotal<<" event..."<<endl;          
    if(_verbosity>1000 && nEvtTotal%5000==0)cout<<"Processed "<<nEvtTotal<<" event..."<<endl;      
    else if(_verbosity>0 && nEvtTotal%50000==0)cout<<"Processed "<<nEvtTotal<<" event..."<<endl;
    
    
    //Your CODE starts here
    //MissingET *met = (MissingET*)brMissingET->At(0); //not using this.
    TLorentzVector mymet(0,0,0,0); // build our own met

    
    readcountmu.clear();
    readcountel.clear();
    readcounttau.clear();
    mu_ptcut.clear();
    el_ptcut.clear();
    mu_ptetacut.clear();
    el_ptetacut.clear();

    
    goodMu.clear(); //clear arrays from previous event
    goodTau.clear();
    jetgen.clear();
    goodEl.clear();
    goodLL.clear();
    predMu.clear();
    predEl.clear();
    predLL.clear();
    predTau.clear();
    eventL.clear();
    goodJet.clear();
    goodEl_temp.clear();
    goodTau_temp.clear();
    cleangoodMu.clear();
    cleangoodEl.clear();
    cleangoodLL.clear();
    cleangoodTau.clear();
    nossf=0;
    nj=0;
    nb=0;
    dr_ll=1.0;
    int nmuons=0; int nparts=0; int nel=0;
    chan_ctr[7]={0};
    LT=0.0; HT=0.0; MET=0.0; LT_MET=0.0; ST=0.0;
    // Tau builder is a function that builds visible hadronic taus from the Gen collection
    TauBuilder();
    // These are genJets. Here we use the Delphes Collection.
    for(int ij=0; ij< brGenJet->GetEntries(); ij++){
      Jet *jt = (Jet*)brGenJet->At(ij);
      Lepton temp; temp.v.SetPtEtaPhiM(jt->PT,jt->Eta,jt->Phi,jt->Mass); temp.ind = ij;
      jetgen.push_back(temp);//put pt eta cuts here
    }
    
    //Now we loop over truth particles and keep what we need.
    for(int no=0; no<brParticle->GetEntries(); no++){
      GenParticle *par = (GenParticle*)brParticle->At(no);
      
      // Building my missing et from neutrinos
      if(par->Status==1){ //its a stable particle
	if(abs(par->PID)==12 || abs(par->PID)==14 || abs(par->PID)==16){//if any BSM signal model also gives a non SM invisible particle(LSP of SUSY, Graviton etc.), their PDGID needs to be added
	  TLorentzVector neut; neut.SetPtEtaPhiM(par->PT,par->Eta,par->Phi,0);
	  mymet += neut;
	}
      }
      
      // Saving muons
      if(par->Status==1 && abs(par->PID)==13){//its a stable muon
	++nmuons;
	Lepton temp; temp.v.SetPtEtaPhiM(par->PT,par->Eta,par->Phi,par->Mass);
	temp.id = par->PID; temp.ind=no; temp.dm=0;
	bool goodMother = false;
	int motherid=1000;
	//debug 1: 
	//cout<<"Reading in Muons: "<<endl<<"PT: "<<temp.v.Pt()<<endl<<"Eta: "<<temp.v.Phi()<<endl<<"Phi: "<<temp.v.Pt()<<endl<<"Mass: "<<temp.v.M()<<endl<<endl;
	readcountmu.push_back(temp);

	
	//Finding mother of muon to check if it is a Tau, or some other Boson/Signal 
	
	do{//do-while loop over the muons origin/lineage to find a non-muon mother 
	  int motherno = par->M1;
	  if(motherno>-1 && motherno<brParticle->GetEntries()){
	    GenParticle *mom = (GenParticle*)brParticle->At(motherno);
	    motherid=mom->PID;
	    par=mom;
	  }
	  else break;
	}while(temp.id == motherid);

	//Now the variable motherid contains the first non-muon mother in the lineage of that muon. Now check if it is boson/signal
	//Include quarks and gluon as the mother in cases when lepton is directly coming from production vertex
	if( abs(motherid)==17 || abs(motherid)==18 || abs(motherid)==23 || abs(motherid)==24 || abs(motherid)==25 || abs(motherid)==42 || abs(motherid)==800018|| abs(motherid)==800020 && !(abs(motherid)==22) ){//tau',nu_tau',W,Z,H,LQ,Seesaw,Not photon
	  temp.mother=0;
	  goodMother=true;
	} 

	//If the first non-muon mother of the muon is a tau, check if the mother of tau is boson/signal
	else if(abs(motherid==15)){
	  int tauid = motherid;//since we dont know if its  tau or antitau
	  
	  do{//do-while loop over the tau origin/lineage to find a non-tau mother 
	    int motherno = par->M1;
	    if(motherno>-1 && motherno<brParticle->GetEntries()){
	      GenParticle *mom = (GenParticle*)brParticle->At(motherno);
	      motherid=mom->PID;
	      par=mom;//in case we have to loop over to find grandmother, greatgrandmother etc...
	    }
	    else break;
	  }while(tauid==motherid);//repeatedly finding mother, grandmother etc... in case the particle is its own mother

	  //Now, motherid contains the first non tau mother of the tau(mother of muon). Check if grandmother of muon is boson/signal

	  if( abs(motherid)==17 || abs(motherid)==18 || abs(motherid)==23 || abs(motherid)==24 || abs(motherid)==25 || abs(motherid)==42 || abs(motherid)==800018|| abs(motherid)==800020 && !(abs(motherid)==22)){//tau',Nu',W,Z,H,LQ,Seesaw,Not Photon
	    temp.mother=1;
	    goodMother=true;
	  }
	  else goodMother=false;
	}
	else goodMother=false; 
	//debug
	if(temp.v.Pt()>5.0){
	  mu_ptcut.push_back(temp);
	}
	if(temp.v.Pt()>5.0 && fabs(temp.v.Eta())<2.4){
	  mu_ptetacut.push_back(temp);
	}
	//Save in goodMu and goodLL array
	if(temp.v.Pt()>5.0 && fabs(temp.v.Eta())<2.4 && goodMother){//Generator Level cuts
	  goodMu.push_back(temp); //store it in the goodMu array.
	  goodLL.push_back(temp);
	  h.mother_lep->Fill(motherid);
	}
      }

      // Saving Electrons (Same process as muons)
      if(par->Status==1 && abs(par->PID)==11){//its a stable electron
	++nel;
	Lepton temp; temp.v.SetPtEtaPhiM(par->PT,par->Eta,par->Phi,par->Mass);
	temp.id = par->PID; temp.ind=no; temp.dm=0;temp.mother=0;
	bool goodMother = false;
	int motherid=1000;

	readcountel.push_back(temp);
	//Finding mother of electron to check if it is a Tau, or some other Boson/Signal 
	
	do{//do-while loop over the muons origin/lineage to find a non-electron mother 
	  int motherno = par->M1;
	  if(motherno>-1 && motherno<brParticle->GetEntries()){
	    GenParticle *mom = (GenParticle*)brParticle->At(motherno);
	    motherid=mom->PID;
	    par=mom;
	  }
	  else break;
	}while(temp.id == motherid);

	//Now the variable motherid contains the first non-electron mother in the lineage of that electron. Now check if it is boson/signal
	
	if( abs(motherid)==17 || abs(motherid)==18 || abs(motherid)==23 || abs(motherid)==24 || abs(motherid)==25 || abs(motherid)==42 || abs(motherid)==800018|| abs(motherid)==800020 && !(abs(motherid)==22) ){//tau',nu_tau',W,Z,H,LQ,Seesaw,Not photon
	  temp.mother=0;
	  goodMother=true;
	} 

	//If the first non-electron mother of the electron is a tau, check if the mother of tau is boson/signal
	else if(abs(motherid==15)){
	  int tauid = motherid;//since we dont know if its  tau or antitau
	  
	  do{//do-while loop over the tau origin/lineage to find a non-tau mother 
	    int motherno = par->M1;
	    if(motherno>-1 && motherno<brParticle->GetEntries()){
	      GenParticle *mom = (GenParticle*)brParticle->At(motherno);
	      motherid=mom->PID;
	      par=mom;//in case we have to loop over to find grandmother, greatgrandmother etc...
	    }
	    else break;
	  }while(tauid==motherid);//repeatedly finding mother, grandmother etc... in case the particle is its own mother

	  //Now, motherid contains the first non tau mother of the tau(mother of electron). Check if grandmother of electron is boson/signal

	  if( abs(motherid)==17 || abs(motherid)==18 || abs(motherid)==23 || abs(motherid)==24 || abs(motherid)==25 || abs(motherid)==42 || abs(motherid)==800018|| abs(motherid)==800020 && !(abs(motherid)==22)){//tau',Nu',W,Z,H,LQ,Seesaw,Not Photon
	    temp.mother=1;
	    goodMother=true;
	  }
	  else goodMother=false;
	}
	else goodMother=false; 

	//debug
	if(temp.v.Pt()>5.0){
	  el_ptcut.push_back(temp);
	}
	if(temp.v.Pt()>5.0 && fabs(temp.v.Eta())<2.4){
	  el_ptetacut.push_back(temp);
	}
	
	//Save in goodEl and goodLL array
	if(temp.v.Pt()>5.0 && fabs(temp.v.Eta())<2.4 && goodMother){//Generator Level cuts
	  goodEl.push_back(temp); //store it in the goodEl array.
	  goodLL.push_back(temp);
	  h.mother_lep->Fill(motherid);
	}
      }
    }//loop over truth particles ends here

    //dR cleaning goodMu, goodEl and goodTau
    Sort(3);//sorting goodMu, goodEl and goodTau (give preference to high pt)

    //We give priority to muons so we clean them first
    if(goodMu.size()==0){
      cleangoodMu.clear();
    }
    else if(goodMu.size()!=0){
      cleangoodMu.push_back(goodMu.at(0));//leading muon always saved
      cleangoodLL.push_back(goodMu.at(0));
      for(int i=1; i<goodMu.size(); i++){//starting with 1, since the 0th leading muon is always saved 
	float dr=9999999.0;
	float dr2=999999.0;
	for(int j=0;j<cleangoodMu.size();j++){
	  dr2=goodMu.at(i).v.DeltaR(cleangoodMu.at(j).v);
	  if(dr2<dr){
	    dr=dr2;
	  }
	}
	if(dr>0.05){
	  cleangoodMu.push_back(goodMu.at(i));
	  cleangoodLL.push_back(goodMu.at(1));
	}
      }
    }
    //Now dR cleaning goodEl. All clean electrons need to be far from all clean muons, so we will check if each goodEl is far from all cleangoodMu's and then save them in goodEl_temp. Then, the first entry in goodEl_temp is the leading electron which will always be selected, and the the remaining need to be checked.
    if(goodEl.size()==0){
      cleangoodEl.clear();
    }
    else if(goodEl.size()!=0 && cleangoodMu.size()!=0){//if cleangoodMu.size()==0; then no electron needs to be checked against any muons, so this step is skipped
      for(int i=0;i<goodEl.size();i++){
	float dr=9999999.0;
	float dr2=999999.0;
	for(int j=0;j<cleangoodMu.size();j++){
	  dr2=goodEl.at(i).v.DeltaR(cleangoodMu.at(j).v);
	  if(dr2<dr){
	    dr=dr2;
	  }
	}
	if(dr>0.05){
	  goodEl_temp.push_back(goodEl.at(i));	  
	}
      }
    }
    if(cleangoodMu.size()==0){
      goodEl_temp=goodEl;
    }

    //Now, the electrons in goodEl_temp are all far away from all cleangoodMu's but they may not be far from each other, so we need to dR clean them with preference to higher pT. 
    if(goodEl_temp.size()==0){
      cleangoodEl.clear();
    }
    else if(goodEl_temp.size()!=0){
      for(int i=0; i<goodEl_temp.size(); i++){
	if(i==0){
	  cleangoodEl.push_back(goodEl_temp.at(i));
	  cleangoodLL.push_back(goodEl_temp.at(i));
	  continue;
	}
	float dr=9999999.0;
	float dr2=999999.0;
	for(int j=0;j<cleangoodEl.size();j++){//check if they are far from the already selected cleangoodEl
	  dr2=goodEl_temp.at(i).v.DeltaR(cleangoodEl.at(j).v);
	  if(dr2<dr){
	    dr=dr2;
	  }
	}
	if(dr>0.05){
	  cleangoodEl.push_back(goodEl_temp.at(i));
	  cleangoodLL.push_back(goodEl_temp.at(i));
	}
      }
    }
    
    //Now, cleangoodMu/El/LL contains all the clean(dR>0.05) muons and electrons.  

    
    //We had called the Taubuilder function above, that returned the goodTau array of Taus that pass certain kinematic cuts, but dR cuts havent been implemented on them yet, we do that next, and save them in cleangoodTau
    //First we dR clean the Taus against all light leptons and create goodTau_temp
    //Then we dR clean the goodTau_temp among themselves with preference to high pT.


    if(goodTau.size()==0){
      cleangoodTau.clear();
    }
    else if(goodTau.size()!=0 && cleangoodLL.size()!=0){//if cleangoodLL.size()==0; then no Tau needs to be checked against any Leptons, so this step is skipped
      for(int i=0;i<goodTau.size();i++){
	float dr=9999999.0;
	float dr2=999999.0;
	for(int j=0;j<cleangoodLL.size();j++){
	  dr2=goodTau.at(i).v.DeltaR(cleangoodLL.at(j).v);
	  if(dr2<dr){
	    dr=dr2;
	  }
	}
	if(dr>0.5){
	  goodTau_temp.push_back(goodTau.at(i));	  
	}
      }
    }
    if(cleangoodLL.size()==0){
      goodTau_temp=goodTau;
    }

    //Now, the Taus in goodTau_temp are all far away from all cleangoodLL's but they may not be far from each other, so we need to dR clean them with preference to higher pT. 
    if(goodTau_temp.size()==0){
      cleangoodTau.clear();
    }
    else if(goodTau_temp.size()!=0){
      for(int i=0; i<goodTau_temp.size(); i++){
	if(i==0){
	  cleangoodTau.push_back(goodTau_temp.at(i));
	  continue;
	}
	float dr=9999999.0;
	float dr2=999999.0;
	for(int j=0;j<cleangoodTau.size();j++){//check if they are far from the already selected cleangoodEl
	  dr2=goodTau_temp.at(i).v.DeltaR(cleangoodTau.at(j).v);
	  if(dr2<dr){
	    dr=dr2;
	  }
	}
	if(dr>0.5){
	  cleangoodTau.push_back(goodTau_temp.at(i));
	}
      }
    }
    
    //Now we have the final cleaned Leptons, so we dR clean the Jets against them.
    
    //dR cleaning of Jets
    if(jetgen.size()!=0){
      for(int i=0;i<jetgen.size();i++){
	float dr=999999.0;
	Lepton temp = jetgen.at(i);
	for(int j=0;j<cleangoodLL.size();j++){//Against light leptons
	  float dr2=temp.v.DeltaR(cleangoodLL.at(j).v);
	  if(dr2<dr){
	    dr = dr2;
	  }
	}
	for(int j=0;j<cleangoodTau.size();j++){//Against Taus(hadronic)
	  float dr2=temp.v.DeltaR(cleangoodTau.at(j).v);
	  if(dr2<dr){
	    dr = dr2;
	  }
	}
	if(dr>0.4 && temp.v.Pt()>30.0 && fabs(temp.v.Eta())<2.4){//cuts on genJets
	  goodJet.push_back(temp);//put pt eta cuts while making jetgen
	  HT+=temp.v.Pt();
	}
      }
    }
    //Assign value of Nj for this event from goodJet array
    nj=goodJet.size();

    //Now we find the number of jets that are b-tagged
    for(int i=0;i<nj; i++){
      nb+= checkisB(goodJet.at(i).v.Pt(),goodJet.at(i).v.Eta(),goodJet.at(i).v.Phi(),goodJet.at(i).v.M());//returns 1 if the jet is b-tagged, else 0
    }//This gives the number of b-jets in the event. 

    //Find  mindR between two light leptons in the event    
    if(cleangoodLL.size()!=0){
      for(int i=0;i<cleangoodLL.size()-1;i++){
	Lepton temp = cleangoodLL.at(i);
	for(int j=i+1;j<cleangoodLL.size();j++){
	  Lepton temp2 = cleangoodLL.at(j);
	  float dr2=temp.v.DeltaR(temp2.v);
	  if(dr2<dr_ll){
	    dr_ll = dr2;
	  }
	}
      }
    }
    if(dr_ll<0.2){
      continue;
    }//If dr_ll<0.2 then event is rejected!! return. If >=0.2 then continue

    //Now we have all variables required to apply efficiency maps
    
    //Applying efficiency maps for all Electrons, Muons and Taus // do in one loop for light leptons
    
    for(int i=0;i<cleangoodEl.size();i++){
      Lepton temp=cleangoodEl.at(i);
      float rndm = rnd->Uniform(0,1);
      float newpt = eff(rndm, temp.v.Pt(),temp.v.Eta(),temp.id,temp.dm,temp.mother,dr_ll,nj,dRmin_OR_Nj);
      Lepton temp2;
      temp2.id=temp.id; temp2.ind=temp.ind; temp2.dm=temp.dm; temp2.wt=temp.wt; temp2.mother=temp.mother; temp2.flavor=temp.flavor;
      if(newpt>-1){
	temp2.v.SetPtEtaPhiM(newpt, temp.v.Eta(), temp.v.Phi(), temp.v.M());
      }
      if(newpt>-1){//check if the object is reconstructed 
	predEl.push_back(temp2); 
	predLL.push_back(temp2);
      }
    } 
    
    for(int i=0;i<cleangoodMu.size();i++){
      Lepton temp=cleangoodMu.at(i);
      float rndm = rnd->Uniform(0,1);
      float newpt = eff(rndm, temp.v.Pt(),temp.v.Eta(),temp.id,temp.dm,temp.mother,dr_ll,nj,dRmin_OR_Nj);
      Lepton temp2;
      temp2.id=temp.id; temp2.ind=temp.ind; temp2.dm=temp.dm; temp2.wt=temp.wt; temp2.mother=temp.mother; temp2.flavor=temp.flavor;
      if(newpt>-1){
	temp2.v.SetPtEtaPhiM(newpt, temp.v.Eta(), temp.v.Phi(), temp.v.M());
      }
      if(newpt>-1){//check if the object is reconstructed 
	predMu.push_back(temp2); 
	predLL.push_back(temp2);
      }
    }
    //do the efficiency map together for mu and el.
    
    for(int i=0;i<cleangoodTau.size();i++){
      Lepton temp=cleangoodTau.at(i);
      float rndm = rnd->Uniform(0,1);
      //cout<<"Tau deets:"<<rndm<<" "<<temp.v.Pt()<<" "<<temp.v.Eta()<<" "<<temp.id<<" "<<temp.dm<<" "<<temp.mother<<" "<<dr_ll<<" "<<nj<<" "<<dRmin_OR_Nj<<endl;
      float newpt = eff(rndm, temp.v.Pt(),temp.v.Eta(),temp.id,temp.dm,temp.mother,dr_ll,nj,dRmin_OR_Nj);
      Lepton temp2;
      temp2.id=temp.id; temp2.ind=temp.ind; temp2.dm=temp.dm; temp2.wt=temp.wt; temp2.mother=temp.mother; temp2.flavor=temp.flavor;
      if(newpt>-1){
	temp2.v.SetPtEtaPhiM(newpt, temp.v.Eta(), temp.v.Phi(), temp.v.M());
      }
      if(newpt>-1){//check if the object is reconstructed // put into same if block
	predTau.push_back(temp2); 
      }
    }
    h.no_predtau->Fill(predTau.size());
    
    //Now, we have all the predicted reco objects.
    //Now we will prepare further classification based on these predicted values.

    
    Sort(1); //sort the predMu predEl predTau and predLL array descending by pT.
    float missinget = mymet.Pt();
    MET=missinget;//save MET 
    h.etmiss->Fill(missinget);//fill a histogram with the missing Et of the event.

    //Now we classify the event into one of the 7 Lepton channels and then furthur subclassify them with respect to nOSSF, pT/MET etc
    bool trigger = (predMu.size()>0 && predMu.at(0).v.Pt()>29.0) || (predEl.size()>0 && predEl.at(0).v.Pt()>35.0);//all events should pass this trigger
      
    if(predLL.size()>=4 && trigger){ // this is 4L
      //iCat = 3;
      chan_ctr[3]++;
      int id[4];
      int k=0;
      
      for(int i=0; i<4; i++){
	eventL.push_back(predLL.at(i));
	LT+=eventL.at(i).v.Pt();//save LT
	id[i] = predLL.at(i).id;
      }

      //All the possible pairs from [0,1,2,3] are stored as boolean variables for invariant mass and ossf check
      //They are then further used for ease of writing the classification code
      
      bool m01 = ((eventL.at(0).v+eventL.at(1).v).M() > 76.0) && ((eventL.at(0).v+eventL.at(1).v).M() < 106.0);
      bool q01 = (id[0]*id[1]==-121) || (id[0]*id[1]==-169);
      bool m02 = ((eventL.at(0).v+eventL.at(2).v).M() > 76.0) && ((eventL.at(0).v+eventL.at(2).v).M() < 106.0);
      bool q02 = (id[0]*id[2]==-121) || (id[0]*id[2]==-169);
      bool m03 = ((eventL.at(0).v+eventL.at(3).v).M() > 76.0) && ((eventL.at(0).v+eventL.at(3).v).M() < 106.0);
      bool q03 = (id[0]*id[3]==-121) || (id[0]*id[3]==-169);
      bool m12 = ((eventL.at(1).v+eventL.at(2).v).M() > 76.0) && ((eventL.at(1).v+eventL.at(2).v).M() < 106.0);
      bool q12 = (id[1]*id[2]==-121) || (id[1]*id[2]==-169);
      bool m13 = ((eventL.at(1).v+eventL.at(3).v).M() > 76.0) && ((eventL.at(1).v+eventL.at(3).v).M() < 106.0);
      bool q13 = (id[1]*id[3]==-121) || (id[1]*id[3]==-169);
      bool m23 = ((eventL.at(2).v+eventL.at(3).v).M() > 76.0) && ((eventL.at(2).v+eventL.at(3).v).M() < 106.0);
      bool q23 = (id[2]*id[3]==-121) || (id[2]*id[3]==-169);
      
      if((q01 && q23) || (q02 && q13) || (q03 && q12)){
	nossf=2;
	if( (m01 && q01 && m23 && q23) || (m02 && q02 && m13 && q13) || (m03 && q03 && m12 && q12)){//double onZ
	  if(nb!=0){//Removing CR contribution
	    ++ctr[33];
	    iCat=33;//category assign D5
	    k=5;
	  }
	  else{
	    chan_ctr[3]-=1;//uncount this event
	    continue;
	  }
	}
	else if(( (m01 && q01) || (m02 && q02) || (m03 && q03) || (m12 && q12) || (m13 && q13) || (m23 && q23) )){//single onZ
	  ++ctr[32];
	  iCat=32;//category assign D4
	  k=4;
	}
	else{ //offZ
	  ++ctr[34];
	  iCat=34;//category assign D6
	  k=6;
	}
      }
      else if( (q01 || q02 || q03 || q12 || q13 || q23) ){
	nossf=1;
	if(( (m01 && q01) || (m02 && q02) || (m03 && q03) || (m12 && q12) || (m13 && q13) || (m23 && q23) )){
	  //single onZ
	  ++ctr[30];
	  iCat=30;//category assign D2
	  k=2;
	}
	else{
	  //offZ
	  ++ctr[31];
	  iCat=31;//category assign D3
	  k=3;
	}
      }
      else{
	nossf=0;
	++ctr[29];
	iCat=29;//category assign D1
	k=1;
      }
      //cout<<id[0]<<" "<<id[1]<<" "<<id[2]<<" "<<id[3]<<" "<<nossf<<" D"<<k<<endl;
      //cout<<m01<<" "<<m02<<" "<<m03<<" "<<m12<<" "<<m13<<" "<<m23<<endl<<endl;
    }// this is 4L
    
    else if(predLL.size()==3 && predTau.size()>=1 && trigger){//3L1T channel
      
      int id[3];
      eventL.push_back(predLL.at(0));
      eventL.push_back(predLL.at(1));
      eventL.push_back(predLL.at(2));
      eventL.push_back(predTau.at(0));
      for(int i=0;i<3;i++){
	id[i]=eventL.at(i).id;
      }
      for(int i=0;i<4;i++){
	LT+=eventL.at(i).v.Pt();//save LT
      }
      bool m01 = ((eventL.at(0).v+eventL.at(1).v).M() > 76.0) && ((eventL.at(0).v+eventL.at(1).v).M() < 106.0);
      bool q01 = (id[0]*id[1]==-121) || (id[0]*id[1]==-169);
      bool m02 = ((eventL.at(0).v+eventL.at(2).v).M() > 76.0) && ((eventL.at(0).v+eventL.at(2).v).M() < 106.0);
      bool q02 = (id[0]*id[2]==-121) || (id[0]*id[2]==-169);
      bool m12 = ((eventL.at(1).v+eventL.at(2).v).M() > 76.0) && ((eventL.at(1).v+eventL.at(2).v).M() < 106.0);
      bool q12 = (id[1]*id[2]==-121) || (id[1]*id[2]==-169);
      
      if( q01 || q02 || q12 ){
	nossf=1;
	if((m01 && q01) || (m02 && q02) || (m12 && q12)){
	  ++ctr[36];
	  iCat=36;//category assign E2
	}
	else{
	  ++ctr[37];
	  iCat=37;//category assign E3
	}
      }
      else{
	nossf=0;
	++ctr[35];
	iCat=35;//category assign E1
      }
      ++chan_ctr[4];
    }//this is 3L1T
    
    else if( predLL.size()==3 && predTau.size()==0 && trigger){//3L event
      //iChan="3L";
      //cout<<"Checking counts of this 3L event: "<<endl;
      //cout<<"No of Muons read in at gen level: "<<readcountmu.size()<<endl;
      //cout<<"No of Electrons read in at gen level: "<<readcountel.size()<<endl;
      //cout<<"No of Taus read in at gen level: "<<readcounttau.size()<<endl;
      //cout<<"No of Muons left after mother cleaning: "<<goodMu.size()<<endl;
      //cout<<"No of Electrons left after mother cleaning: "<<goodEl.size()<<endl;      
      //cout<<"No of Taus left after mother cleaning: "<<goodTau.size()<<endl;
      //cout<<"No of Muons left after dR cleaning: "<<cleangoodMu.size()<<endl;
      //cout<<"No of Electrons left after dR cleaning: "<<cleangoodEl.size()<<endl;
      //cout<<"No of Taus left after dR cleaning: "<<cleangoodTau.size()<<endl;
      //cout<<"No of Muons left after effmap: "<<predMu.size()<<endl;
      //cout<<"No of Electrons left after effmap: "<<predEl.size()<<endl;
      //cout<<"No of Taus left after effmap: "<<predTau.size()<<endl<<endl;
      
      if(readcountmu.size()+readcountel.size()==3){
	notseeningen++;
      }
      else if(mu_ptcut.size()+el_ptcut.size()==3){
	lostinptcut++;
      }
      else if((mu_ptetacut.size()+el_ptetacut.size())==3){
	lostinptetacut++;
      }
      else if((goodMu.size()+goodEl.size())==3){
	lostingoodmother++;
      }
      else if((cleangoodMu.size()+cleangoodEl.size())==3){
	lostindr++;
      }
      else{
	lostineff++;
      }
      
	
      chan_ctr[0]++;
      int id[3]={0};
      eventL.push_back(predLL.at(0));
      eventL.push_back(predLL.at(1));
      eventL.push_back(predLL.at(2));
      for(int i=0;i<3;i++){
	id[i]=eventL.at(i).id;
	//cout<<id[i]<<" ";
	LT+=eventL.at(i).v.Pt();//save LT
      }
      h.etmiss3l->Fill(MET);

      float mass01 = (eventL.at(0).v+eventL.at(1).v).M();
      bool m01 = (mass01 > 76.0) && (mass01 < 106.0);
      bool q01 = (id[0]*id[1]==-121) || (id[0]*id[1]==-169);

      float mass02 = (eventL.at(0).v+eventL.at(2).v).M();
      bool m02 = (mass02 > 76.0) && (mass02 < 106.0);
      bool q02 = (id[0]*id[2]==-121) || (id[0]*id[2]==-169);

      float mass12 = (eventL.at(1).v+eventL.at(2).v).M();
      bool m12 = (mass12 > 76.0) && (mass12< 106.0);
      bool q12 = (id[1]*id[2]==-121) || (id[1]*id[2]==-169);
      
      float masst0 = sqrt(2*eventL.at(0).v.Pt()*MET*(1-cos(delta_phi(eventL.at(0).v.Phi(),mymet.Phi()))));
      bool mt0 = masst0>50.0 && masst0<150.0;
      float masst1 = sqrt(2*eventL.at(1).v.Pt()*MET*(1-cos(delta_phi(eventL.at(1).v.Phi(),mymet.Phi()))));
      bool mt1 = masst1>50.0 && masst1<150.0;
      float masst2 = sqrt(2*eventL.at(2).v.Pt()*MET*(1-cos(delta_phi(eventL.at(2).v.Phi(),mymet.Phi()))));
      bool mt2 = masst2>50.0 && masst2<150.0;

      bool trilepmassonZ = (eventL.at(0).v+eventL.at(1).v+eventL.at(2).v).M()>=76.0 && (eventL.at(0).v+eventL.at(1).v+eventL.at(2).v).M()<=106.0;
      bool ptcut = eventL.at(2).v.Pt()>25.0;//since the eventL array is sorted by pT, the 3rd lepton must have the lowest pT.
      float st_ = LT+HT+MET;
      bool isnb_greq_one = false;
      if(nb>=1){
	bool isnb_greq_one = true;
      }

      //cout<<"Event Details unconditional: "<<endl<<"Count in 3L: "<<chan_ctr[0]<<endl<<"IDs: "<<id[0]<<" "<<id[1]<<" "<<id[2]<<endl<<"Mass01: "<<mass01<<endl<<"M01: "<<m01<<endl<<"Q01: "<<q01<<endl<<"Mass02: "<<mass02<<endl<<"M02: "<<m02<<endl<<"Q02: "<<q02<<endl<<"Mass12: "<<mass12<<endl<<"M12: "<<m12<<endl<<"Q12: "<<q12<<endl<<"Masst0: "<<masst0<<endl<<"MT0: "<<mt0<<endl<<"Masst1: "<<masst1<<endl<<"MT1: "<<mt1<<endl<<"Masst2: "<<masst2<<endl<<"MT2: "<<mt2<<endl<<"TrilepmassonZ: "<<trilepmassonZ<<endl<<"PTcut: "<<ptcut<<endl<<"ST: "<<st_<<endl<<"number of bjets>=1?: "<<isnb_greq_one<<endl<<"MET: "<<MET<<endl<<endl;
      
      if(q01 || q02 || q12){//atleast one ossf pair
	nossf=1;
	if(abs(id[0]*id[1]*id[2])!=1331 && abs(id[0]*id[1]*id[2])!=2197){//event is unambiguous (not eee/mumumu)
	  //find the unambiguous pair
	  if(q01){//01 is the ossf pair
	    if(m01){//01 is onZ pair
	      if(masst2>125.0){
		if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){//remove 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue;
		}
		else{
		  ++ctr[8];
		  iCat=8; //assign A9
		}
	      }
	      else{
		if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){//remove 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue;
		}
		else{
		  ++ctr[2];
		  iCat=2; //assign A3
		}
	      }
	    }
	    else if(mass01<76.0){//the unambiguous pair is belowZ mass window
	      cout<<"pT3 value: "<<eventL.at(2).v.Pt()<<endl;
	      if(ptcut){
		if(nb==0 && trilepmassonZ){//removing 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue;
		}
		else{ 
		  ++ctr[9];
		  iCat=9; //assign A10
		}
	      }
	      else{
		if(nb==0 && trilepmassonZ){//removing 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue;
		}
		else{
		  ++ctr[3];
		  iCat=3; //assign A4
		}
	      }
	    }
	    else{//the unambiguous pair is aboveZ mass window
	      if(ptcut){
		++ctr[10];
		iCat=10; //assign A11
	      }
	      else{
		++ctr[4];
		iCat=4; //assign A5
	      }
	    }
	  }
	  else if(q02){//02 is the ossf pair
	    if(m02){//02 is onZ     
	      if(masst1>125.0){
		if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		  chan_ctr[0]--;
		  continue; 
		}
		else{
		  ++ctr[8];
		  iCat=8; //assign A9
		}
	      }
	      else{
		if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		  chan_ctr[0]--;
		  continue; 
		}
		else{
		  ++ctr[2];
		  iCat=2; //assign A3
		}
	      }
	    }
	    else if(mass02<76.0){//the unambiguous pair is belowZ mass window
	      cout<<"pT3 value: "<<eventL.at(2).v.Pt()<<endl;
	      if(ptcut){
		if(nb==0 && trilepmassonZ){//remove 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue; 
		}
		else{
		  ++ctr[9];
		  iCat=9; //assign A10
		}
	      }
	      else{
	        if(nb==0 && trilepmassonZ){//removing 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue; 
		}
		else{
		  ++ctr[3];
		  iCat=3; //assign A4
		}
	      }
	    }
	    else{//the unambiguous pair is aboveZ mass window
	      if(ptcut){
		++ctr[10];
		iCat=10; //assign A11
	      }
	      else{
		++ctr[4];
		iCat=4; //assign A5
	      }
	    }
	  }
	  else{//12 is the ossf pair
	    if(m12){//12 is onZ
	      if(masst0>125.0){
		if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		  chan_ctr[0]--;
		  continue; 
		}
		else{
		  ++ctr[8];
		  iCat=8; //assign A9
		}
	      }
	      else{
		if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		  chan_ctr[0]--;
		  continue; 
		}
		else{
		  ++ctr[2];
		  iCat=2; //assign A3
		}
	      }
	    }
	    else if(mass12<76.0){//the unambiguous pair is belowZ mass window
	      cout<<"pT3 value: "<<eventL.at(2).v.Pt()<<endl;
	      if(ptcut){
		if(nb==0 && trilepmassonZ){//removing 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue; 
		}
		else{
		  ++ctr[9];
		  iCat=9; //assign A10
		}
	      }
	      else{
	        if(nb==0 && trilepmassonZ){//removing 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue; 
		}
		else{
		  ++ctr[3];
		  iCat=3; //assign A4
		}
	      }
	    }
	    else{//the unambiguous pair is aboveZ mass window
	      if(ptcut){
		++ctr[10];
		iCat=10; //assign A11
	      }
	      else{
		++ctr[4];
		iCat=4; //assign A5
	      }
	    }
	  }
	}
	else{//the event is ambiguous (eee/mumumu)
	  //We need to find which pair comes from the Z and which comes from W
	  if(q01 && q02){//the opposite sign lepton is at 0
	    if( (m01 && mt2)||(m02 && mt1) ){//onZ event
	      if( (m01 && mt2) && !(m02 && mt1)){ 
		
		if(masst2>125.0){
		  if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		    chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[8];
		    iCat=8; //assign A9
		  }
		}
		else{
		  if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		    chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[2];
		    iCat=2; //assign A3
		  }
		}	
	      }
	      else if(!(m01 && mt2) && (m02 && mt1)){
	        if(masst1>125.0){
		  if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[8];
		    iCat=8; //assign A9
		  }
		}
		else{
		  if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		    chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[2];
		    iCat=2; //assign A3
		  }
		}
	      }
	      else{//both possibilities are satisfied, check which pair has invm closer to Z pole mass
		if(abs(mass01-91.0)<=abs(mass02-91.0)){//m01 is the Z pair
		  if(masst2>125.0){
		    if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[8];
		      iCat=8; //assign A9
		    }
		  }
		  else{
		    if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[2];
		      iCat=2; //assign A3
		    }
		  }
		}
		else{//m02 is the Z pair
		  if(masst1>125.0){
		    if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[8];
		      iCat=8; //assign A9
		    }
		  }
		  else{
		    if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[2];
		      iCat=2; //assign A3
		    }
		  }
		}
	      }
	    }
	    else if(mass01<76.0 && mass02<76.0){
	      cout<<"pT3 value: "<<eventL.at(2).v.Pt()<<endl;
	      if(ptcut){
		if(nb==0 && trilepmassonZ){//remove 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue; 
		}
		else{
		  ++ctr[9];
		  iCat=9; //assign A10
		}
	      }
	      else{
		if(nb==0 && trilepmassonZ){//remove 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue;
		}
		else{
		  ++ctr[3];
		  iCat=3; //assign A4
		}
	      }
	    }
	    else if(mass01>106.0 && mass02>106.0){//aboveZ
	      if(ptcut){
		++ctr[10];
		iCat=10; //assign A11
	      }
	      else{
		++ctr[4];
		iCat=4; //assign A5
	      }
	    }
	    else{//mixedZ
	      if(ptcut){
		++ctr[11];
		iCat=11; //assign A12
	      }
	      else{
		++ctr[5];
		iCat=5; //assign A6
	      }
	    }
	  }
	  //else
	  else if(q01 && q12){//the opposite sign lepton is at 1
	    if( (m01 && mt2)||(m12 && mt0) ){//onZ event
	      if( (m01 && mt2) && !(m12 && mt0)){
	        if(masst2>125.0){
		  if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		    chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[8];
		    iCat=8; //assign A9
		  }
		}
		else{
		  if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		    chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[2];
		    iCat=2; //assign A3
		  }
		}
	      }
	      else if(!(m01 && mt2) && (m12 && mt0)){
	        if(masst0>125.0){
		  if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		    chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[8];
		    iCat=8; //assign A9
		  }
		}
		else{
		  if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		    chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[2];
		    iCat=2; //assign A3
		  }
		}
	      }
	      else{//both possibilities are satisfied, check which pair has invm closer to Z pole mass
		if(abs(mass01-91.0)<=abs(mass12-91.0)){//m01 is the Z pair
		  if(masst2>125.0){
		    if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[8];
		      iCat=8; //assign A9
		    }
		  }
		  else{
		    if( (MET<125.0 && masst2<150.0)||(nb==0 && MET<100.0 && masst2<50.0)||(nb==0 && MET<125.0 && mt2 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst2<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[2];
		      iCat=2; //assign A3
		    }
		  }
		}
		else{//m12 is the Z pair
		  if(masst0>125.0){
		    if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[8];
		      iCat=8; //assign A9
		    }
		  }
		  else{
		    if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[2];
		      iCat=2; //assign A3
		    }
		  }
		}
	      }
	    }
	    else if(mass01<76.0 && mass12<76.0){
	      cout<<"pT3 value: "<<eventL.at(2).v.Pt()<<endl;
	      if(ptcut){
		if(nb==0 && trilepmassonZ){//remove 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue;
		}
		else{
		  ++ctr[9];
		  iCat=9; //assign A10
		} 
	      }
	      else{
		if(nb==0 && trilepmassonZ){//remove 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue;
		}
		else{
		  ++ctr[3];
		  iCat=3; //assign A4
		}
	      }
	    }
	    else if(mass01>106.0 && mass12>106.0){
	      if(ptcut){
		++ctr[10];
		iCat=10; //assign A11
	      }
	      else{
		++ctr[4];
		iCat=4; //assign A5
	      }
	    }
	    else{//mixedZ
	      if(ptcut){
		++ctr[11];
		iCat=11; //assign A12
	      }
	      else{
		++ctr[5];
		iCat=5; //assign A6
	      }
	    }
	  }
	  //else
	  else if(q02 && q12){//the opposite sign lepton is at 2
	    if( (m02 && mt1)||(m12 && mt0) ){//onZ event
	      if( (m02 && mt1) && !(m12 && mt0)){
	        if(masst1>125.0){
		  if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[8];
		    iCat=8; //assign A9
		  }
		}
		else{
		  if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[2];
		    iCat=2; //assign A3
		  }
		}
	      }
	      else if(!(m02 && mt1) && (m12 && mt0)){
	        if(masst0>125.0){
		  if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[8];
		    iCat=8; //assign A9
		  }
		}
		else{
		  if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		    continue;
		  }
		  else{
		    ++ctr[2];
		    iCat=2; //assign A3
		  }
		}
	      }
	      else{//both possibilities are satisfied, check which pair has invm closer to Z pole mass
		if(abs(mass02-91.0)<=abs(mass12-91.0)){//m02 is the Z pair
		  if(masst1>125.0){
		    if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		        chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[8];
		      iCat=8; //assign A9
		    }
		  }
		  else{
		    if( (MET<125.0 && masst1<150.0)||(nb==0 && MET<100.0 && masst1<50.0)||(nb==0 && MET<125.0 && mt1 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst1<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		        chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[2];
		      iCat=2; //assign A3
		    }
		  }
		}
		else{//m12 is the Z pair
		  if(masst0>125.0){
		    if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		        chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[8];
		      iCat=8; //assign A9
		    }
		  }
		  else{
		    if( (MET<125.0 && masst0<150.0)||(nb==0 && MET<100.0 && masst0<50.0)||(nb==0 && MET<125.0 && mt0 && (eventL.at(2).v.Pt()>20.0))||(isnb_greq_one && (MET<125.0) && (masst0<150.0) && (eventL.at(2).v.Pt()>20.0) && nj>2 && st_>350.0)){
		      chan_ctr[0]--;
		      continue;
		    }
		    else{
		      ++ctr[2];
		      iCat=2; //assign A3
		    }
		  }
		}
	      }
	    }
	    else if(mass02<76.0 && mass12<76.0){
	      cout<<"pT3 value: "<<eventL.at(2).v.Pt()<<endl;
	      if(ptcut){
		if(nb==0 && trilepmassonZ){//remove 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue;
		}
		else{
		  ++ctr[9];
		  iCat=9; //assign A10
		} 
	      }
	      else{
		if(nb==0 && trilepmassonZ){//remove 3L CR
		  chan_ctr[0]--;//uncount this event
		  continue;
		}
		else{
		  ++ctr[3];
		  iCat=3; //assign A4
		}
	      }
	    }
	    else if(mass02>106.0 && mass12>106.0){
	      if(ptcut){
		++ctr[10];
		iCat=10; //assign A11
	      }
	      else{
		++ctr[4];
		iCat=4; //assign A5
	      }
	    }
	    else{//mixedZ
	      if(ptcut){
		++ctr[11];
		iCat=11; //assign A12
	      }
	      else{
		++ctr[5];
		iCat=5; //assign A6
	      }
	    }
	  }
	}	
      }
      else{//ossf0 event
	nossf=0;
	if( (id[0]>0 && id[1]>0 && id[2]>0) || (id[0]<0 && id[1]<0 && id[2]<0)){//same sign event
	  if(ptcut){
	    ++ctr[7];
	    iCat=7; //assign A8
	  }
	  else{
	    ++ctr[1];
	    iCat=1;  //assign A2
	  }
	}
	else{//not same sign event
	  if(ptcut){
	    ++ctr[6];
	    iCat=6; //assign A7
	  }
	  else{
	    ++ctr[0];
	    iCat=0; //assign A1
	  }
	}
      }
    }//this is 3L
    else if(predLL.size()==2 && predTau.size()>=2 && trigger){//2L2T
      //iChan="2L2T";
      
      int id[4];
      eventL.push_back(predLL.at(0));
      eventL.push_back(predLL.at(1));
      eventL.push_back(predTau.at(0));
      eventL.push_back(predTau.at(1));
      for(int i=0;i<4;i++){
	id[i]=eventL.at(i).id;
	LT+=eventL.at(i).v.Pt();//save LT
      }
      bool m01 = ((eventL.at(0).v+eventL.at(1).v).M() > 76.0) && ((eventL.at(0).v+eventL.at(1).v).M() < 106.0);
      bool q01 = (id[0]*id[1]==-121) || (id[0]*id[1]==-169);
      bool m23 = ((eventL.at(2).v+eventL.at(3).v).M() > 76.0) && ((eventL.at(2).v+eventL.at(3).v).M() < 106.0);
      bool q23 = (id[2]*id[3]==-121) || (id[2]*id[3]==-169);
      if(q01 && q23){
	nossf=2;
	if(m01 && q01){
	  ++ctr[40];
	  iCat=40;//category assign F3
	}
	else{
	  ++ctr[41];
	  iCat=41;//category assign F4
	}
      }
      else if(q01 || q23){
	nossf=1;
	++ctr[39];
	iCat=39;//category assign F2
    	
      }
      else{
	nossf=0;
	++ctr[38];
	iCat=38;//category assign F1
      }
      ++chan_ctr[5];
    }//this is 2L2T
    else if(predLL.size()==2 && predTau.size()==1 && trigger){//2L1T
      //iChan="2L1T";
      ++chan_ctr[1];
      int id[3];
      eventL.push_back(predLL.at(0)); id[0]=eventL.at(0).id;
      eventL.push_back(predLL.at(1)); id[1]=eventL.at(1).id;
      eventL.push_back(predTau.at(0));id[2]=eventL.at(2).id;
      for(int i=0;i<3;i++){
	LT+=eventL.at(i).v.Pt();//save LT
      }
      bool high = eventL.at(2).v.Pt()>50.0;
      
      if(id[0]*id[1]==-121 || id[0]*id[1]==-169){
	nossf=1;
	float m01= (eventL.at(0).v+eventL.at(1).v).M();
	if(m01>106.0){
	  if(high){
	    ++ctr[23];
	    iCat=23;//category assign B12
	  }
	  else{
	    ++ctr[17];
	    iCat=17;//category assign B6
	  }
	}
	else if(m01<76.0){
	  if(high){
	    ++ctr[22];
	    iCat=22;//category assign B11
	  }
	  else{
	    ++ctr[16];
	    iCat=16;//category assign B5
	  }
	}
	else{
	  if(MET>100.0){//removing CR region from 2L1T
	    if(high){
	      ++ctr[21];
	      iCat=21;//category assign B10
	    }
	    else{
	      ++ctr[15];
	      iCat=15;//category assign B4
	    }
	  }
	  else{
	    --chan_ctr[1];
	    continue;
	  }
	}
      }
      else{
	nossf=0;
	float m02 = (eventL.at(0).v+eventL.at(2).v).M();//first DF pair
	bool q02 = (id[0]*id[2]<0);//is OS?
	float m12 = (eventL.at(1).v+eventL.at(2).v).M();//second DF pair
	bool q12 = (id[1]*id[2]<0);//is OS?
	float m01=0.0;
	bool q01=false;
	//Now, the two light leptons may also be DF, so we need to check that too
	if(abs(id[0])!=abs(id[1])){//the two light leptons are also DF
	  m01 = (eventL.at(0).v+eventL.at(1).v).M();//third DF pair
	  q01 = (id[0]*id[1]<0);//is OS?
	}
	float mass=0.0;
	if(q02){
	  mass=m02;
	}
	if(q12 && m12>mass){ 
	  mass=m12;
	}
	if(q02 && m02>mass){
	  mass=m02;
	}
	if(mass>91.0){
	  if(high){
	    ++ctr[19];
	    iCat=19;//category assign B8
	  }
	  else{
	    ++ctr[13];
	    iCat=13;//category assign B2
	  }
	}
	else{
	  if(high){
	    ++ctr[18];
	    iCat=18;//category assign B7
	  }
	  else{
	    ++ctr[12];
	    iCat=12;//category assign B1
	  }
	}
	if(!q02 && !q12 && !q01){
	  if(high){
	    ++ctr[20];
	    iCat=20;//category assign B9
	  }
	  else{
	    ++ctr[14];
	    iCat=14;//category assign B3
	  }
	}
      }
    }//this is 2LT
    else if(predLL.size()==1 && predTau.size()>=3 && trigger){//1L3T
      //iChan="1L3T";
      ++chan_ctr[6];
      eventL.push_back(predLL.at(0));
      eventL.push_back(predTau.at(0));
      eventL.push_back(predTau.at(1));
      eventL.push_back(predTau.at(2));
      for(int i=0;i<4;i++){
	LT+=eventL.at(i).v.Pt();//save LT
      }
      ++ctr[42];
      iCat=42;//category assign G1
      
    }
    else if(predLL.size()==1 && predTau.size()==2 && trigger){//1L2T
      //iChan="1L2T";
      chan_ctr[2]++;
      int id[3];
      eventL.push_back(predLL.at(0));  id[0]=eventL.at(0).id;
      eventL.push_back(predTau.at(0)); id[1]=eventL.at(1).id;
      eventL.push_back(predTau.at(1)); id[2]=eventL.at(2).id;
      for(int i=0;i<3;i++){
	LT+=eventL.at(i).v.Pt();//save LT
      }
      if(id[1]*id[2]==-225){//tau OS pair
	nossf=1;
	float m12=(eventL.at(1).v+eventL.at(2).v).M();
	if(m12>91.0){
	  ctr[28]++;
	  iCat=28;//category assign C5
	}
	else{
	  ctr[27]++;
	  iCat=27;//category assign C4
	}
      }
      else{
	nossf=0;//this means that both the taus have the same sign, so an OSDF pair will only exist if the light lepton has opposite sign from both the taus.
	float m01 = (eventL.at(0).v+eventL.at(1).v).M();
	bool q01 = (id[0]*id[1]<0);
	float m02 = (eventL.at(0).v+eventL.at(2).v).M();
	bool q02 = (id[0]*id[2]<0);
	float mass=0.0;
	if(q01){
	  mass=m01;
	}
	if(q02 && m02>mass){
	  mass=m02;
	}
	if(mass>91.0){
	  ctr[25]++;
	  iCat=25;//category assign C2
	}
	else{
	  ctr[24]++;
	  iCat=24;//category assign C1
	}
	if(!q01 && !q02){
	  ctr[26]++;
	  iCat=26;//category assign C3
	}
      }
    }
    else continue;
    
    LT_MET=LT+MET;
    ST=LT_MET+HT;
    h.summary[0][iCat]->Fill(LT);
    h.summary[1][iCat]->Fill(HT);
    h.summary[2][iCat]->Fill(MET);
    h.summary[3][iCat]->Fill(ST);
    h.summary[4][iCat]->Fill(LT_MET);
     
  }//end loop over events.
  cout<<"Not seen in gen: "<<notseeningen<<endl;
  cout<<"Lost in ptcut: "<<lostinptcut<<endl;
  cout<<"Lost in etacut: "<<lostinptetacut<<endl;
  cout<<"Lost in goodmother cut"<<lostingoodmother<<endl;
  cout<<"Lost in dR: "<<lostindr<<endl;
  cout<<"Lost in eff: "<<lostineff<<endl;

  //

  cout<<"N_ZZ/WZ = "<<endl;
  for(int i=0;i<12;i++){
    cout<<"N_"<<iCat_name[i]<<": "<<138*(1000)*(0.09462)*(ctr[i])/20000<<endl;
  }
  return;
}


int delZAna::checkisB(float pt,float eta, float phi, float m){
  float mybtageff=1.0;
  Lepton jet; jet.v.SetPtEtaPhiM(pt,eta,phi,m);
  float dr=99999.0;
  for(int i=0;i<brParticle->GetEntries(); i++){
     GenParticle *par = (GenParticle*)brParticle->At(i);
     if( (par->PID>500 && par->PID<600)||(par->PID>5000 && par->PID<6000) ){
       Lepton temp_par; temp_par.v.SetPtEtaPhiM(par->PT,par->Eta,par->Phi,par->Mass);
       temp_par.id = par->PID; temp_par.ind=i; 
       float dr2 = jet.v.DeltaR(temp_par.v);
       if(dr2<dr){
	 dr=dr2;
       }
     }
  }
  float rndm = rnd->Uniform(0,1);
  if(dr<0.4){//found a b hadron inside 0.4dR cone
    if(rndm<=mybtageff){//b tagging efficiency
      return 1;
    }
    else{
      return 0;
    }
  }
  else{
    return 0;
  }
}


void delZAna::Sort(int opt)
{
  //Sort selected objects by pT (always descending).
  //option 1 sorts the predMu, predEl, predTau, predLL arrays
  if(opt==1){
    if(predMu.size()!=0){
      for(int i=0; i<(int)predMu.size()-1; i++){
	for(int j=i+1; j<(int)predMu.size(); j++){
	  if( predMu[i].v.Pt() < predMu[j].v.Pt() ) swap(predMu.at(i),predMu.at(j)); }}}
    if(predTau.size()!=0){
      for(int i=0; i<(int)predTau.size()-1; i++){
	for(int j=i+1; j<(int)predTau.size(); j++){
	  if( predTau[i].v.Pt() < predTau[j].v.Pt() ) swap(predTau.at(i),predTau.at(j)); }}}
    if(predEl.size()!=0){
    for(int i=0; i<(int)predEl.size()-1; i++){
      for(int j=i+1; j<(int)predEl.size(); j++){
	if( predEl[i].v.Pt() < predEl[j].v.Pt() ) swap(predEl.at(i),predEl.at(j)); }}}
    if(predLL.size()!=0){
      for(int i=0; i<(int)predLL.size()-1; i++){
	for(int j=i+1; j<(int)predLL.size(); j++){
	  if( predLL[i].v.Pt() < predLL[j].v.Pt() ) swap(predLL.at(i),predLL.at(j)); }}}
    
  }
  if(opt==2){
    //Sorts the GenJets
    for(int i=0; i<(int)jetgen.size()-1; i++){
      for(int j=i+1; j<(int)jetgen.size(); j++){
	if( jetgen[i].v.Pt() < jetgen[j].v.Pt() ) swap(jetgen.at(i),jetgen.at(j)); }}
  }
  if(opt==3){
     if(goodMu.size()!=0){
      for(int i=0; i<(int)goodMu.size()-1; i++){
	for(int j=i+1; j<(int)goodMu.size(); j++){
	  if( goodMu[i].v.Pt() < goodMu[j].v.Pt() ) swap(goodMu.at(i),goodMu.at(j)); }}}
    if(goodTau.size()!=0){
      for(int i=0; i<(int)goodTau.size()-1; i++){
	for(int j=i+1; j<(int)goodTau.size(); j++){
	  if( goodTau[i].v.Pt() < goodTau[j].v.Pt() ) swap(goodTau.at(i),goodTau.at(j)); }}}
    if(goodEl.size()!=0){
    for(int i=0; i<(int)goodEl.size()-1; i++){
      for(int j=i+1; j<(int)goodEl.size(); j++){
	if( goodEl[i].v.Pt() < goodEl[j].v.Pt() ) swap(goodEl.at(i),goodEl.at(j)); }}}
  }
    //If you have other arrays, write code here to sort them.
}
float delZAna::delta_phi(float phi1, float phi2)//Find separation in phi space
{
  phi1 = TVector2::Phi_0_2pi(phi1);
  phi2 = TVector2::Phi_0_2pi(phi2);
  float dphi = fabs(phi1 - phi2);
  if(dphi>TMath::Pi()) dphi = 2*TMath::Pi() - dphi;
  return dphi;
}


Double_t delZAna::delR(TLorentzVector v1,  TLorentzVector v2)//Find angular separation in eta-phi space
{ 
  double del_eta = abs(v1.Eta() - v2.Eta());
  return sqrt( pow(del_eta,2) + pow(delta_phi(v1.Phi(),v2.Phi()),2) );
}
float delZAna::calc_iso(float dR, TLorentzVector v1, int skip)
{
  // Sum the pT of all stable particles with deltaR<dR of particle v1
  // Skip the particle itself (skip) and skip neutrinos.
  float isol = 0;
  for(int no=0; no<brParticle->GetEntries(); no++){
    GenParticle *par = (GenParticle*)brParticle->At(no);
    if(no==skip) continue;
    if(par->Status==1){
      if(abs(par->PID)==12 || abs(par->PID)==14 || abs(par->PID)==16) continue;
      TLorentzVector v2; v2.SetPtEtaPhiM(par->PT,par->Eta,par->Phi,par->Mass);
      if(v1.DeltaR(v2)<dR) isol+=v2.Pt();
    }
  }
  return isol;

}


float delZAna::eff(float x, float genpt, float geneta, int pdgid, int tauDM=1, int mother=0, float drmin=1, float nj=2, int dRmin_OR_Nj=1)
{

  if(abs(pdgid)!=11 && abs(pdgid)!=13 && abs(pdgid)!=15){
    cout<<"Please enter correct PDG ID of the leptons (+/-11, +/-13 or +/-15)"<<endl;
    return 0;
  }
  if((tauDM!=1 && tauDM!=3) && abs(pdgid)==15){
    cout<<"Please enter correct decay mode of hadronic tau lepton (DM=1 for 1-prong, DM=3 for 3-prong)"<<endl;
    return 0;
  }
  if(mother!=0 && mother!=1){
    cout<<"Please enter correct mother information of the leptons (mother=0 for light leptons from bosons or signal particles, and mother=1 for light leptons from taus)"<<endl;
    return 0;
  }
  if(dRmin_OR_Nj!=0 && dRmin_OR_Nj!=1){
    cout<<"Please enter correct global event parametrization you want to use (dRmin_OR_Nj=0 for dRmin based parametrization and dRmin_OR_Nj=1 for Nj based parametrization)"<<endl;
    return 0;
  }

  h.grndm->Fill(x);
    
  float llbarrelpt[24] = {0,5,10,15,20,25,30,35,40,50,60,70,80,100,150,200,250,300,350,400,450,500,750,1000};
  float llendcappt[23] = {0,5,10,15,20,25,30,35,40,50,60,70,80,100,150,200,250,300,350,400,450,500,1000};
  float tau1pbarrelpt[22] = {0,15,20,25,30,35,40,50,60,70,80,100,150,200,250,300,350,400,450,500,750,1000};
  float tau1pendcappt[18] = {0,15,20,25,30,35,40,50,60,70,80,100,150,200,250,350,450,1000};
  float tau3pbarrelpt[22] = {0,15,20,25,30,35,40,50,60,70,80,100,150,200,250,300,350,400,450,500,750,1000};
  float tau3pendcappt[18] = {0,15,20,25,30,35,40,50,60,70,80,100,150,200,250,350,450,1000};
  float llbarrelpt_tauMom[20] = {0,5,10,15,20,25,30,35,40,50,60,70,80,100,150,200,300,400,500,1000};
  float llendcappt_tauMom[19] = {0,5,10,15,20,25,30,35,40,50,60,70,80,100,150,200,300,500,1000};

  TH2F *hist;
  int binN = 0;
  float pt = -1;
  vector<float> effVal, pTVal;
  TString plotname;
  
  if(dRmin_OR_Nj==0){ //DR parametrization
    if(abs(pdgid)==11){ //electron
      if(fabs(geneta)<=1.1 && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_barrelpt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_barrelpt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_barrelpt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_barrelpt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if((fabs(geneta)>1.1 && fabs(geneta)<=1.6) && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_transitionpt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_transitionpt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_transitionpt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_transitionpt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(fabs(geneta)>1.6 && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_endcappt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_endcappt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_endcappt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_endcappt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
    }
    
    if(abs(pdgid)==13){ //muon
      if(fabs(geneta)<=1.2 && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Muon_barrelpt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Muon_barrelpt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Muon_barrelpt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Muon_barrelpt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(fabs(geneta)>1.2 && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Muon_endcappt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Muon_endcappt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Muon_endcappt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Muon_endcappt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
    }

    if(abs(pdgid)==15){ //tau
      if(tauDM==1){ //1-prong
	if(fabs(geneta)<=1.1){
	  if(drmin>0.2){
	    plotname = "Tau1p_barrelpt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0]); i++){
	      if(i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i] && genpt<tau1pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pbarrelpt[i]+tau1pbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.1 && fabs(geneta)<=1.6){
	  if(drmin>0.2){
	    plotname = "Tau1p_transitionpt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.6){
	  if(drmin>0.2){
	    plotname = "Tau1p_endcappt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(tauDM==3){ //3-prong
	if(fabs(geneta)<=1.1){
	  if(drmin>0.2){
	    plotname = "Tau3p_barrelpt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0]); i++){
	      if(i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i] && genpt<tau3pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pbarrelpt[i]+tau3pbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.1 && fabs(geneta)<=1.6){
	  if(drmin>0.2){
	    plotname = "Tau3p_transitionpt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }          
	}
	if(fabs(geneta)>1.6){
	  if(drmin>0.2){
	    plotname = "Tau3p_endcappt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }      
	}
      }
    }
  }
  
  if(dRmin_OR_Nj==1){ //Nj parametrization
    if(abs(pdgid)==11){ //electron
      if(fabs(geneta)<=1.1 && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Electron_barrelpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_barrelpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Electron_barrelpt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_barrelpt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if((fabs(geneta)>1.1 && fabs(geneta)<=1.6) && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Electron_transitionpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_transitionpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Electron_transitionpt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_transitionpt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(fabs(geneta)>1.6 && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Electron_endcappt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_endcappt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Electron_endcappt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_endcappt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
    }
    
    if(abs(pdgid)==13){ //muon
      if(fabs(geneta)<=1.2 && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Muon_barrelpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Muon_barrelpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Muon_barrelpt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Muon_barrelpt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(fabs(geneta)>1.2 && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Muon_endcappt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Muon_endcappt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Muon_endcappt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Muon_endcappt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
    }

    if(abs(pdgid)==15){ //tau
      if(tauDM==1){ //1-prong
	if(fabs(geneta)<=1.1){
	  if(nj<2){
	    plotname = "Tau1p_barrelpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0]); i++){
	      if(i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i] && genpt<tau1pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pbarrelpt[i]+tau1pbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Tau1p_barrelpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0]); i++){
	      if(i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i] && genpt<tau1pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pbarrelpt[i]+tau1pbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.1 && fabs(geneta)<=1.6){
	  if(nj<2){
	    plotname = "Tau1p_transitionpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Tau1p_transitionpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.6){
	  if(nj<2){
	    plotname = "Tau1p_endcappt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Tau1p_endcappt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(tauDM==3){ //3-prong
	if(fabs(geneta)<=1.1){
	  if(nj<2){
	    plotname = "Tau3p_barrelpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0]); i++){
	      if(i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i] && genpt<tau3pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pbarrelpt[i]+tau3pbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Tau3p_barrelpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0]); i++){
	      if(i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i] && genpt<tau3pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pbarrelpt[i]+tau3pbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.1 && fabs(geneta)<=1.6){
	  if(nj<2){
	    plotname = "Tau3p_transitionpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }          
	  if(nj>1){
	    plotname = "Tau3p_transitionpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }          
	}
	if(fabs(geneta)>1.6){
	  if(nj<2){
	    plotname = "Tau3p_endcappt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }      
	  if(nj>1){
	    plotname = "Tau3p_endcappt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }      
	}
      }
    }
  }
  
  float effsum = 0.00001;
  
  for(int j=0; j<(int)effVal.size(); j++){
    if(x>=effsum && x<(effVal[j]+effsum)){
      pt = pTVal[j];
      break;
    }
    effsum += effVal[j];
  }      

  return pt;
}


void delZAna::TauBuilder()//call after building goodLL
{
  //First we get the list of taus and keep them in tauind
  // we only keep those taus which dont decay to leptons
  // i.e. we want only those taus which decay hadronically
  vector<int> tauind;
  int motherid; int ctr2=0;
  int cd=0;//charged daughters
  for(int no=0; no<brParticle->GetEntries(); no++){
    GenParticle *par = (GenParticle*)brParticle->At(no);
    if( abs(par->PID)==15 ){
      bool gottau=true;
      ctr2++;
      bool goodMother=false;
      int pid=par->PID;
      motherid=1000;

      //Removing Taus which decay leptonically
      for(int da=par->D1;da<par->D2+1;da++){
	GenParticle *parda = (GenParticle*)brParticle->At(da);
	if(abs(parda->PID)==15 || abs(parda->PID)==11 || abs(parda->PID)==13){
	  gottau=false; break;
	}
      }
      Lepton tautemp; tautemp.v.SetPtEtaPhiM(par->PT,par->Eta,par->Phi,par->Mass);
      readcounttau.push_back(tautemp);
      
      do{//removing taus with tau mothers
	int motherno = par->M1;
	if(motherno>-1 && motherno<brParticle->GetEntries()){
	  GenParticle *mom = (GenParticle*)brParticle->At(motherno);
	  motherid=mom->PID;
	  par=mom;//in case we have to loop over to find grandmother, greatgrandmother etc...
	}
	else break;
      }while(pid == motherid);
      
      if(abs(motherid)==17 || abs(motherid)==18 || abs(motherid)==23 || abs(motherid)==24 || abs(motherid)==25 || abs(motherid)==42 || abs(motherid)==800018 || abs(motherid)==800020 && !(abs(motherid)==22)){//tau',Nu',W,Z,H,LQ,Seesaw
	goodMother=true;
      }
      
      
      if(gottau && goodMother) tauind.push_back(no);
    }
  }
  //cout<<(int)tauind.size()<<endl;
  
  // Now for the taus which decay hadronically, we
  // loop over the visible daughters (dav) and build the visible 4-vector (vistau)
  // We only keep those taus with pT>20 GeV in goodTau.
  for(int i=0; i<(int)tauind.size(); i++){
    cd=0;
    GenParticle *tau = (GenParticle*)brParticle->At(tauind.at(i));
    TLorentzVector vistau(0,0,0,0);
    for(int da=tau->D1;da<tau->D2+1;da++){
      GenParticle *dt = (GenParticle*)brParticle->At(da);
      if((dt->Charge)!=0) cd++;
      if(abs(dt->PID)>10 && abs(dt->PID)<17) continue;
      //cout<<"NonLeptonic Daughter"<<endl;
      TLorentzVector dav; dav.SetPtEtaPhiM(dt->PT,dt->Eta,dt->Phi,dt->Mass);
      vistau += dav;
    }
    Lepton temp; temp.v = vistau; temp.id=tau->PID; temp.ind=tauind.at(i);
    if(cd==1)temp.dm=1;//assign decay mode
    else temp.dm=3;

    temp.mother=0;
    if(temp.v.Pt()>15 && fabs(temp.v.Eta())<2.3){
      goodTau.push_back(temp);
      h.mother_lep->Fill(motherid);
    }
  }
  h.no_gentau->Fill(ctr2);
}
void delZAna::BookHistograms()
{
  // Booking syntax for histogram pointers (that we declared in struct Hists in header)
  // obj = new CLASS("localname","Histogram title",NumberOfBins,LowerEdge,HigherEdge)

  h.etmiss3l = new TH1F("etmiss3l","Missing E_{T} in 3L",500,0,500); h.etmiss3l->Sumw2();
  h.etmiss = new TH1F("etmiss","Missing E_{T}",500,0,500); h.etmiss->Sumw2();
  h.mother_lep = new TH1F("mother_lep","Mother of Selected Leptons",1200,-600,600);
  h.grndm = new TH1F("random_no","Generated Random Number",10,0,1);
  h.no_gentau = new TH1F("no_gentau","No of Gen Taus",15,0,15);
  h.no_predtau = new TH1F("no_predtau","No of Pred Taus",15,0,15);
  
  //h.summary[6][43] //43 categories, 6 distributions for each // {"A1","A2","A3","A4","A5","A6","A7","A8","A9","A10","A11","A12","B1","B2","B3","B4","B5","B6","B7","B8","B9","B10","B11","B12","C1","C2","C3","C4","C5","D1","D2","D3","D4","D5","D6","E1","E2","E3","F1","F2","F3","F4","G1"};
  TString crname[5] = {"LT","HT","MET","ST","LTMET"};
  TString plotname[43] = {"A1","A2","A3","A4","A5","A6","A7","A8","A9","A10","A11","A12","B1","B2","B3","B4","B5","B6","B7","B8","B9","B10","B11","B12","C1","C2","C3","C4","C5","D1","D2","D3","D4","D5","D6","E1","E2","E3","F1","F2","F3","F4","G1"};
  int nbins=10;
  float blo=0.0;
  float bhi= 2000.0;
  for(int icr=0; icr<5; icr++){//gen
    for(int iplot=0; iplot<43; iplot++){
      TString name = crname[icr] + "_" + plotname[iplot];
      h.summary[icr][iplot] = new TH1F(name,name,nbins,blo,bhi); h.summary[icr][iplot]->Sumw2();
    }
  }
  
  // Note the calling of Sumw2() for each histogram we declare.
  // This is needed so that if scale the histograms later, the
  // errors on the points are still correct after scaling.
  // This is relevant.. so ask someone if you don't understand.
  
}
